function [yQ, dYdXQ, gap] = loess(x, y, xQ, xWindow, doFill)
% loess estimates local regression estimate at specified x values
% Arguments:
% x, y: data (vectors)
% xQ: specified points for estimation
% yQ, dYdXQ: smoothed estimates of y and dYdX at xQ points (vectors)
% xWindow: size of window along x axis (scalar)
% doFill: true causes function to fill nan gaps (scalar, default: false)
% gap: nan or 1, indicating missing or present estimates (vector)
% The moving average calculation will return a nan where there are gaps
% in the input data larger than the span indicated by xWindow.
% If doFill==true, then those nans are replaced by values interpolated
% from adjacent points in the data series.
%
% Mark Brandon, Yale University, 2020

%% Initialize variables
if length(x)~=length(y)
    error('x and y vectors must be the same length')
end
if any(isnan(xQ)) || ~isvector(xQ)
    error('xQ argument must be a vector without no nan values.');
end
if nargin<5, doFill=false; end

%... Preprocess x, y, xQ
% Make sure that x, y, xQ are column vectors
x = x(:); y = y(:); xQ = xQ(:);
% Remove any nans from x,y, if present
isNum = ~isnan(x) & ~isnan(y);
x = x(isNum);
y = y(isNum);
% Find data with same x values, and remove by averaging y values
[x,~,i] = unique(x,'stable');
y = accumarray(i, y, [], @mean);

%% Compute
%... Estimate yQ value for specified xQ values
nQ = length(xQ);
yQ = nan(nQ,1);
dYdXQ = nan(nQ,1);
dYdXQQ = nan(nQ,1);
for i = 1:nQ
    % Dimensionless distance from current xQ value
    u = (x - xQ(i))/(xWindow/2);
    j = abs(u)<1;
    nJ = sum(j);
    if nJ>2
        w = (1-abs(u(j)).^3).^3;  % weight function for x distance
        %V = w.*[ones(nJ,1), u(j), u(j).^2];
        V = w.*[ones(nJ,1), u(j)];
        % Estimate weighted fit for 2nd-order polynomial using QR method
        [Q,R] = qr(V,0);
        b = R\(Q'*(w.*y(j)));
        yQ(i) = b(1);
        dYdXQ(i) = 2*b(2)/xWindow;
        %dYdXQQ(i) = 2*b(3)/xWindow;
    end
end

% Define a vector, gap, that has elements set to 1 or nan, which indicate 
% if or if not the associated yQ value is a loess estimate. 
isGap = isnan(yQ);
gap = double(~isGap);
gap(gap==0) = nan;
% If doFill==true, then missing yQ and dYdXQ values are estimated by 
% interpolation from adjacent yQ and dYdXQ values. 
% If doFill==false, then the missing values are returned as nans. 
if doFill==true && any(isGap)
    yQ(isGap) = interp1(xQ(~isGap), yQ(~isGap), xQ(isGap));
    dYdXQ(isGap) = interp1(xQ(~isGap), dYdXQ(~isGap), xQ(isGap));    
end

